/**
Provides a web worker process. 
**/

// ID for console output
var consoleID = 0;

/**
 * Convert an array buffer to a string.
 *
 * @method ab2str
 * @param  {ArrayBuffer} buf A raw buffer to convert to string, 2 bytes per char, (endian sensitive!).
 * @return {String}      The string representation of the buffer. 
function ab2str(buf) {
    return String.fromCharCode.apply(null, new Uint16Array(buf));
}
 */

/**
 * Convert a string to an array buffer.
 *
 * @method str2ab
 * @param  {String} str A string to convert to an array buffer.
 * @return {String} The buffer representation of the string, 2 bytes per char, (endian sensitive!). 
function str2ab(str) {
    var buf = new ArrayBuffer(str.length * 2); // 2 bytes for each char
    var bufView = new Uint16Array(buf);
    for (var i = 0, strLen = str.length; i < strLen; i++) {
        bufView[i] = str.charCodeAt(i);
    }
    return buf;
}
 */

/**
 * Send progress update to parent process.
 *
 * @method reportProgress
 * @param  {Number} value Percentage of progress so far.
 */
function reportProgress (value) {
    self.postMessage({msgType: 'progress', 'id': consoleID, 'value': value });
}

/**
 * Send a log message to the parent process.
 *
 * @method log
 * @param  {String} s The log message text.
 */
function log (s) {
    self.postMessage({msgType: 'log', 'id': consoleID, string: s });
}

self.addEventListener('message', function(e) {
    var data = e.data;
    console.log(data.msgType);
    switch (data.msgType) {
        case 'init_openspin':
            consoleID = data.id;
            ModuleConf['arguments'] = data.args;
            log ('w Worker started - consoleID: ' + data.id);
            reportProgress(10);
            openspin(ModuleConf);
            log ('w OpenSpin run time env. created.');
            reportProgress (20);
            // now we are playing pingpong with the calling process. execution continues with messages 'put_file' followed by a 'call_openspin'. 
            self.postMessage({msgType: 'init_openspin_done', 'id': consoleID});            
            break;
        case 'put_file':
            log ('w Writing ' + data.name + ' to MemFS.');
            try {
                //ModuleConf.FS_createDataFile('', data.name, ab2str(data.buffer), true, true, true);
                ModuleConf.FS_createDataFile('', data.name, data.buffer, true, true, true);
            } catch (e) {
                log ('w Exception put_file: ' + e );
            }
            break;
        case 'call_openspin':             
            reportProgress (50);
            log ('w Running openspin main');
            ModuleConf['callMain'](ModuleConf['arguments']);    // Call openspin's main method. 
            log ('w Finished openspin main.');
            reportProgress (80);
            // now we are playing pingpong with the calling process. execution continues with a message 'get_file' followed by a 'terminate'. 
            self.postMessage({msgType: 'call_openspin_done', 'id': consoleID});            
            break;
        case 'get_file':
            log ('w Reading ' + data.name + ' from MemFS.');
            var u8b=new Uint8Array();
            try {
                u8b = ModuleConf['FS'].readFile(data.name, { encoding: 'binary' });
            } catch (e) {
                log ('w No such file.');
            }
            reportProgress (90);//get_file_done
            self.postMessage({msgType: 'get_file_done', 'id': consoleID, 'buffer': u8b});     
            break;
        case 'terminate':
            reportProgress (100);
            self.postMessage({msgType: 'terminating', 'id': consoleID});
            self.close();    // Terminates the worker.
            break;
        default:
            self.postMessage({msgType: "unknown", 'id': consoleID});
    };
}, false);


var ModuleConf = {};

ModuleConf['preRun'] = function () {
//    log ('preRun:');
};

ModuleConf['postRun'] = function () {
//    log ('postRun:');
};

ModuleConf['print'] = function print(s) {
    log ('+ ' + s);
};

ModuleConf['printErr'] = function printErr(s) {
    log ('- ' + s);
};
 
ModuleConf['arguments'] = ['-b', 'test.spin']; 

// Let's not have main execute immeditaly on load.
ModuleConf['noInitialRun'] = true;

// TBD: What to do with this?
ModuleConf['noExitRuntime'] = true;

// Pull in the openspin compiler
importScripts('openspin.js');

